--[[
Love Benchmark by easy
]]--


-- List all the tests here:
require "texttest"
require "circletest"
require "spritetest"
require "balltest"
require "bugtest"
tests = { textTest, circleTest, spriteTest, ballTest, bugTest }


local lg = love.graphics
local lk = love.keyboard
local lt = love.timer
local le = love.event

local fontHeight
local videoModes = {}
local activeMode = 1
local changeStep = 0
local activeTest = 1
local debugLines = 0
local debugWidth = 0
local drawDebug = true


function love.load()
	-- Start up
	targetFPS = 60
	targetDelta = 1 / targetFPS
	currentFPS = 0
	currentDelta = 0

	-- Set the font
    font = lg.newFont("data/ubuntu.ttf", 20)
	lg.setFont(font)
	fontHeight = font:getHeight("Ág")

	-- Get videomodes
	videoModes = lg.getModes()
	table.sort(videoModes, function(a, b) return a.width * a.height < b.width * b.height end)
	screenWidth, screenHeight, fullscreen, vsync, fsaa = lg.getMode()

	-- Find the active video mode on the list
	for	i = 1, #videoModes do
		if screenWidth == videoModes[i].width and screenHeight == videoModes[i].height then
			activeMode = i
			break
		end
	end

	-- Load test related
	for i = 1, #tests do tests[i].load() end
end

function love.keyreleased(key)
	-- Quit if neccessary
	if key == "escape" then le.quit() end

	-- Change resoltuion
	local changeRes = 0
	if key == "f1" then changeRes = -1 end
	if key == "f2" then changeRes = 1 end
	
	if changeRes ~= 0 then
		local a = activeMode + changeRes
		a = a > #videoModes and 1 or a
		a = a < 1 and #videoModes or a

		local w = videoModes[a].width
		local h = videoModes[a].height

		if w ~= screenWidth and h ~= screenHeight and lg.checkMode(w, h, fullscreen) then
			lg.setMode(w, h, fullscreen, vsync, fsaa)
			screenWidth = w
			screenHeight = h
			activeMode = a
		end
	end

	-- Toggle fullscreen
	if key == "f3" and lg.toggleFullscreen() then fullscreen = not fullscreen end

	-- Toggle vsync
	if key == "f4" then
		local v = not vsync
		if lg.setMode(screenWidth, screenHeight, fullscreen, v, fsaa) then vsync = v end
	end

	-- Toggle antialiasing
	if key == "f5" then
		local a = fsaa
		a = a == 0 and 2 or a * 2
		a = a > 8 and 0 or a

		if lg.setMode(screenWidth, screenHeight, fullscreen, vsync, a) then fsaa = a end
	end

	-- Next test
	if key == "right" then
		activeTest = activeTest + 1
		activeTest = activeTest > #tests and 1 or activeTest
	end

	-- Previous test
	if key == "left" then
		activeTest = activeTest - 1
		activeTest = activeTest < 1 and #tests or activeTest
	end

	-- Increase target FPS
	if key == "up" then
		targetFPS = targetFPS + 10
		targetDelta = 1 / targetFPS
	end

	-- Decrease target FPS
	if key == "down" then
		targetFPS = targetFPS - 10
		targetFPS = targetFPS < 10 and 10 or targetFPS
		targetDelta = 1 / targetFPS
	end

	-- Hide panel
	if key == "h" then drawDebug = not drawDebug end
end

function love.update(dt)
	currentFPS = 1 / dt
	currentDelta = dt

	-- Update the active test
	tests[activeTest].update(dt)
end

function love.draw()
	-- Perform the test
	tests[activeTest].draw()

	if drawDebug then
		-- Draw debug box
		lg.setColor(0, 0, 0, 160)
		lg.rectangle("fill", 0, 0, debugWidth + 20, fontHeight * debugLines + 20)
	
		-- Draw debug info
		lg.setFont(font)
		debugLines = 0
		debugWidth = 0
		printDebug(tests[activeTest].name, activeTest, "Lt/Rg")
		printDebug("Debug panel", tostring(drawDebug), "H")
		printDebug("Resolution", string.format("%d x %d", screenWidth, screenHeight), "F1/F2")
		printDebug("Fullscreen", tostring(fullscreen), "F3")
		printDebug("VSync", tostring(vsync), "F4")
		printDebug("FSAA", fsaa, "F5")
		printDebug("Current FPS", string.format("%d", currentFPS))
		printDebug("Target FPS", string.format("%d", targetFPS), "Up/Dn")
		printDebug("Current delta", string.format("%.3f", currentDelta))
		printDebug("Target delta", string.format("%.3f", targetDelta))
		printDebug("Change step", string.format("%d", changeStep))
		tests[activeTest].debug()
	end
end

-- Prints debug info on the screen
function printDebug(name, value, keys)
	local pos = 0
	local width = 0
	
	-- Print keys
	if keys then
		local t = "[" .. keys .. "]"
		local w = font:getWidth(t)

		lg.setColor(255, 0, 0, 255)
		lg.print(t, 10 + pos, 10 + fontHeight * debugLines)

		pos = pos + w + 10
		width = width + w + 10
	end

	-- Print variable name
	if name then
		local t = name .. ":"
		local w = font:getWidth(t)

		lg.setColor(255, 255, 255, 255)
		lg.print(t, 10 + pos, 10 + fontHeight * debugLines)

		pos = pos + w + 10
		width = width + w + 10
	end

	-- Print variable value
	if value then
		local w = font:getWidth(value)

		lg.setColor(0, 255, 0, 255)
		lg.print(value, 10 + pos, 10 + fontHeight * debugLines)

		pos = pos + w
		width = width + w
	end

	debugWidth = width > debugWidth and width or debugWidth
	debugLines = debugLines + 1
end

-- Changes the number of samples in order to meet the target FPS
function synchronize(value)
	changeStep = 0

	-- Try to meet target FPS
	if currentDelta ~= targetDelta then
		local tl = 0.01		-- Large threshold
		local tm = 0.005	-- Medium threshold
		local ts = 0		-- Small threshold
		local sl = 100		-- Large step
		local sm = 10		-- Medium step
		local ss = 1		-- Small step

		-- How much to add or substract?
		if 		currentDelta < targetDelta - tl then changeStep = sl
		elseif 	currentDelta < targetDelta - tm then changeStep = sm
		elseif 	currentDelta < targetDelta - ts then changeStep = ss
		elseif 	currentDelta > targetDelta + tl then changeStep = -sl
		elseif	currentDelta > targetDelta + tm then changeStep = -sm
		elseif	currentDelta > targetDelta + ts then changeStep = -ss
		end

		-- Apply changes
		value = value + changeStep

		-- Don't go under 1
		if value < 1 then value = 1 end
	end

	return value
end
